@@ -1,5 +1,5 @@
 #
-# Copyright (c), 2018-2020, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2021, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -19,9 +19,9 @@ import os.path
 import unicodedata
 from copy import copy
 from decimal import Decimal, DecimalException
+from string import ascii_letters
 from urllib.parse import urlsplit, quote as urllib_quote
 
-from ..exceptions import ElementPathTypeError
 from ..datatypes import QNAME_PATTERN, DateTime10, DateTime, Date10, Date, \
     Float10, DoubleProxy, Time, Duration, DayTimeDuration, YearMonthDuration, \
     UntypedAtomic, AnyURI, QName, NCName, Id, is_idrefs, ArithmeticProxy, NumericProxy
@@ -36,6 +36,11 @@ from .xpath2_parser import XPath2Parser
 method = XPath2Parser.method
 function = XPath2Parser.function
 
+
+def is_local_url_scheme(scheme):
+    return scheme in ('', 'file') or len(scheme) == 1 and scheme in ascii_letters
+
+
 ###
 # Sequence types (allowed only for type checking in treat-as/instance-of statements)
 function('empty-sequence', nargs=0, label='sequence type')
@@ -119,17 +124,14 @@ def select(self, context=None):
         yield from self.parser.namespaces
     elif hasattr(elem, 'nsmap'):
         # For lxml returns Element's prefixes
-        if 'xml' not in elem.nsmap:
-            yield 'xml'
-        yield from filter(lambda x: x, elem.nsmap)
+        yield 'xml'
+        yield from filter(lambda x: x and x != 'xml', elem.nsmap)
     else:
         # For ElementTree returns module registered prefixes
-        prefixes = {x for x in self.parser.namespaces if x}
+        prefixes = {'xml'}  # 'xml' prefix is always registered
+        prefixes.update(x for x in self.parser.namespaces if x)
         if context.namespaces:
             prefixes.update(x for x in context.namespaces if x)
-
-        if 'xml' not in prefixes:
-            yield 'xml'
         yield from prefixes
 
 
@@ -187,26 +189,16 @@ def evaluate(self, context=None):
     name = node_name(arg)
     if name is None:
         return
-
-    if name.startswith('{'):
+    elif name.startswith('{'):
+        # name is a QName in extended format
         namespace, local_name = split_expanded_name(name)
         for pfx, uri in self.parser.namespaces.items():
             if uri == namespace:
-                if pfx:
-                    return QName(uri, '{}:{}'.format(pfx, local_name))
-                else:
-                    return QName(uri, local_name)
+                return QName(uri, '{}:{}'.format(pfx, local_name))
         raise self.error('FONS0004', 'no prefix found for namespace {}'.format(namespace))
-
-    try:
-        if ':' not in name:
-            return QName(self.parser.namespaces.get('', ''), name)
-        pfx, _ = name.strip().split(':')
-        return QName(self.parser.namespaces[pfx], name)
-    except ValueError:
-        raise self.error('FORG0001', 'invalid value {!r} for argument'.format(name.strip()))
-    except KeyError as err:
-        raise self.error('FONS0004', 'no namespace found for prefix {}'.format(err))
+    else:
+        # name is a local name
+        return QName(self.parser.namespaces.get('', ''), name)
 
 
 @method(function('nilled', nargs=1))
@@ -378,8 +370,6 @@ def evaluate(self, context=None):
         if isinstance(item, UntypedAtomic):
             values.append(self.cast_to_double(item))
             float_class = float
-        elif isinstance(item, (DayTimeDuration, Date10, YearMonthDuration, str, int, Decimal)):
-            values.append(item)
         elif isinstance(item, float):
             values.append(item)
             if float_class is None:
@@ -388,6 +378,8 @@ def evaluate(self, context=None):
                 float_class = float
         elif isinstance(item, AnyURI):
             values.append(item.value)
+        elif isinstance(item, (DayTimeDuration, YearMonthDuration)):
+            values.append(item)
         elif isinstance(item, (Duration, QName)):
             raise self.error('FORG0006', "xs:{} is not an ordered type".format(type(item).name))
         else:
@@ -520,22 +512,16 @@ def select(self, context=None):
 
     if len(self) == 2:
         for pos, result in enumerate(self[0].select(context), start=1):
-            try:
-                if starting_loc <= pos:
-                    yield result
-            except TypeError as err:
-                raise self.error('XPTY0004', err) from None
+            if starting_loc <= pos:
+                yield result
     else:
         length = self.get_argument(context, 2, cls=NumericProxy)
         if not math.isnan(length) and not math.isinf(length):
             length = round(length)
 
         for pos, result in enumerate(self[0].select(context), start=1):
-            try:
-                if starting_loc <= pos < starting_loc + length:
-                    yield result
-            except TypeError as err:
-                raise self.error('XPTY0004', err) from None
+            if starting_loc <= pos < starting_loc + length:
+                yield result
 
 
 @method(function('unordered', nargs=1))
@@ -615,16 +601,14 @@ def evaluate(self, context=None):
                             return False
 
                     elif isinstance(value2, bool):
-                        if not isinstance(value1, bool) or value1 is not value2:
-                            return False
+                        return False
 
                     elif isinstance(value1, UntypedAtomic):
                         if not isinstance(value2, UntypedAtomic) or value1 != value2:
                             return False
 
                     elif isinstance(value2, UntypedAtomic):
-                        if not isinstance(value1, UntypedAtomic) or value1 != value2:
-                            return False
+                        return False
 
                     elif isinstance(value1, float):
                         if math.isnan(value1):
@@ -638,8 +622,7 @@ def evaluate(self, context=None):
 
                     elif isinstance(value2, float):
                         if math.isnan(value2):
-                            if not math.isnan(value1):
-                                return False
+                            return False
                         elif isinstance(value1, Decimal):
                             if value2 != float(value1):
                                 return False
@@ -789,17 +772,20 @@ def evaluate(self, context=None):
 
 ###
 # String functions
-def xml10_chr(cp):
-    if cp in {0x9, 0xA, 0xD} \
-            or 0x20 <= cp <= 0xD7FF \
-            or 0xE000 <= cp <= 0xFFFD \
-            or 0x10000 <= cp <= 0x10FFFF:
-        return chr(cp)
-    raise ValueError("{} is not a valid XML 1.0 codepoint".format(cp))
-
 
 @method(function('codepoints-to-string', nargs=1))
 def evaluate(self, context=None):
+
+    def xml10_chr(cp: int):
+        if not isinstance(cp, int):
+            raise TypeError("invalid type {} for codepoint {}".format(type(cp), cp))
+        elif cp in {0x9, 0xA, 0xD} \
+                or 0x20 <= cp <= 0xD7FF \
+                or 0xE000 <= cp <= 0xFFFD \
+                or 0x10000 <= cp <= 0x10FFFF:
+            return chr(cp)
+        raise ValueError("{} is not a valid XML 1.0 codepoint".format(cp))
+
     try:
         return ''.join(xml10_chr(cp) for cp in self[0].select(context))
     except TypeError as err:
@@ -821,14 +807,8 @@ def evaluate(self, context=None):
 def evaluate(self, context=None):
     comp1 = self.get_argument(context, 0, cls=str, promote=(AnyURI, UntypedAtomic))
     comp2 = self.get_argument(context, 1, cls=str, promote=(AnyURI, UntypedAtomic))
-    if not isinstance(comp1, str):
-        if comp1 is None:
-            return
-        comp1 = str(comp1)
-    if not isinstance(comp2, str):
-        if comp2 is None:
-            return
-        comp2 = str(comp2)
+    if comp1 is None or comp2 is None:
+        return
 
     if len(self) < 3:
         value = locale.strcoll(comp1, comp2)
@@ -866,12 +846,7 @@ def evaluate(self, context=None):
 @method(function('string-join', nargs=(1, 2)))
 def evaluate(self, context=None):
     items = [self.string_value(s) for s in self[0].select(context)]
-    try:
-        return self.get_argument(context, 1, required=True, cls=str).join(items)
-    except ElementPathTypeError:
-        raise
-    except TypeError as err:
-        raise self.error('XPTY0004', "the values must be strings: %s" % err) from None
+    return self.get_argument(context, 1, required=True, cls=str).join(items)
 
 
 @method(function('normalize-unicode', nargs=(1, 2)))
@@ -960,8 +935,6 @@ def evaluate(self, context=None):
 def evaluate(self, context=None):
     arg1 = self.get_argument(context, default='', cls=str)
     arg2 = self.get_argument(context, index=1, default='', cls=str)
-    if arg1 is None:
-        return ''
 
     if len(self) < 3:
         index = arg1.find(arg2)
@@ -1253,11 +1226,8 @@ def evaluate(self, context=None):
 @method(function('element-with-id', nargs=(1, 2)))
 @method(function('id', nargs=(1, 2)))
 def select(self, context=None):
-    try:
-        idrefs = {x for item in self[0].select(copy(context))
-                  for x in self.string_value(item).split()}
-    except AttributeError:
-        raise self.error('XPTY0004', "1st argument must returns strings")
+    idrefs = {x for item in self[0].select(copy(context))
+              for x in self.string_value(item).split()}
 
     node = self.get_argument(context, index=1, default_to_context=True)
     if isinstance(context, XPathSchemaContext):
@@ -1357,7 +1327,8 @@ def evaluate(self, context=None):
         except (KeyError, TypeError):
             if self.symbol == 'doc':
                 url_parts = urlsplit(uri)
-                if url_parts.scheme in ('', 'file') and os.path.isdir(url_parts.path.lstrip(':')):
+                if is_local_url_scheme(url_parts.scheme) \
+                        and os.path.isdir(url_parts.path.lstrip(':')):
                     raise self.error('FODC0005', 'document URI is a directory')
                 raise self.error('FODC0002')
             return False
@@ -1396,7 +1367,7 @@ def evaluate(self, context=None):
             collection = context.collections[uri]
         except (KeyError, TypeError):
             url_parts = urlsplit(uri)
-            if url_parts.scheme in ('', 'file') and \
+            if is_local_url_scheme(url_parts.scheme) and \
                     not url_parts.path.startswith(':') and url_parts.path.endswith('/'):
                 raise self.error('FODC0003', 'collection URI is a directory')
             raise self.error('FODC0002', '{!r} collection not found'.format(uri)) from None
@@ -1441,7 +1412,7 @@ def evaluate(self, context=None):
 def select(self, context=None):
     label = self.get_argument(context, index=1, cls=str)
     for value in self[0].select(context):
-        '{} {}'.format(label, str(value).strip())  # TODO
+        '{} {}'.format(label, str(value).strip())  # TODO: trace dataset
         yield value
 
 
