@@ -16,8 +16,9 @@ try:
 except ImportError:
     lxml_etree = None
 
-from elementpath import AttributeNode, XPathContext, XPath2Parser
+from elementpath import AttributeNode, XPathContext, XPath2Parser, MissingContextError
 from elementpath.namespaces import XML_LANG, XSD_NAMESPACE
+from elementpath.schema_proxy import AbstractXsdSchema
 
 try:
     # noinspection PyPackageRequirements
@@ -56,8 +57,20 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
 
     def setUp(self):
         self.schema_proxy = XMLSchemaProxy(self.schema)
-        self.parser = XPath2Parser(namespaces=self.namespaces, schema=self.schema_proxy,
-                                   variables=self.variables)
+        self.parser = XPath2Parser(namespaces=self.namespaces, schema=self.schema_proxy)
+
+    def test_abstract_xsd_schema(self):
+        class XsdSchema(AbstractXsdSchema):
+            @property
+            def attrib(self):
+                return {}
+
+            def __iter__(self):
+                return iter(())
+
+        schema = XsdSchema()
+        self.assertEqual(schema.tag, '{http://www.w3.org/2001/XMLSchema}schema')
+        self.assertIsNone(schema.text)
 
     def test_schema_proxy_init(self):
         schema_src = """<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
@@ -88,7 +101,8 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
         self.wrong_syntax("schema-element(*)")
         self.wrong_name("schema-element(nil)")
         self.wrong_name("schema-element(xs:string)")
-        self.check_value("self::schema-element(xs:complexType)", [])
+        self.check_value("self::schema-element(xs:complexType)", MissingContextError)
+        self.check_value("self::schema-element(xs:complexType)", NameError, context)
         self.check_value("self::schema-element(xs:schema)", [context.item], context)
         self.check_tree("schema-element(xs:group)", '(schema-element (: (xs) (group)))')
 
@@ -96,12 +110,17 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
         self.wrong_syntax("schema-attribute(*)")
         self.wrong_name("schema-attribute(nil)")
         self.wrong_name("schema-attribute(xs:string)")
-        self.check_value("self::schema-attribute(xml:lang)", [])
-        self.check_select("schema-attribute(xml:lang)", [])
+        self.check_value("self::schema-attribute(xml:lang)", MissingContextError)
+        self.check_value("schema-attribute(xml:lang)", NameError, context)
         self.check_value("self::schema-attribute(xml:lang)", [context.item], context)
         self.check_tree("schema-attribute(xsi:schemaLocation)",
                         '(schema-attribute (: (xsi) (schemaLocation)))')
 
+        token = self.parser.parse("self::schema-attribute(xml:lang)")
+        context.item = AttributeNode(XML_LANG, 'en')
+        context.axis = 'attribute'
+        self.assertEqual(list(token.select(context)), [context.item])
+
     def test_bind_parser_method(self):
         schema_src = """<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
                             <xs:simpleType name="test_type">
@@ -149,6 +168,8 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
         any_simple_type = schema_proxy.get_type('{%s}anySimpleType' % XSD_NAMESPACE)
         self.assertEqual(schema_proxy.get_primitive_type(any_simple_type), any_simple_type)
 
+    @unittest.skipIf(xmlschema is None or xmlschema.__version__ < '1.2.3',
+                     "Old find API does not work")
     def test_find_api(self):
         schema_src = """<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
                             <xs:element name="test_element" type="xs:string"/>
@@ -238,11 +259,13 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
         self.assertEqual(token[0].xsd_types['b'], schema.maps.types['{%s}integer' % XSD_NAMESPACE])
         token = parser.parse("//values/c")
 
-        self.assertEqual(token[0][0].xsd_types["values"], schema.elements['values'].type)
+        self.assertEqual(token[0][0].xsd_types["{http://xpath.test/ns}values"],
+                         schema.elements['values'].type)
         self.assertEqual(token[1].xsd_types['c'], schema.maps.types['{%s}boolean' % XSD_NAMESPACE])
 
         token = parser.parse("values/c")
-        self.assertEqual(token[0].xsd_types['values'], schema.elements['values'].type)
+        self.assertEqual(token[0].xsd_types['{http://xpath.test/ns}values'],
+                         schema.elements['values'].type)
         self.assertEqual(token[1].xsd_types['c'], schema.maps.types['{%s}boolean' % XSD_NAMESPACE])
 
         token = parser.parse("values/*")
@@ -284,7 +307,8 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
         self.assertEqual(token[0].xsd_types['b'], schema.types['rangeType'])
         token = parser.parse("values/c")
 
-        self.assertEqual(token[0].xsd_types['values'], schema.elements['values'].type)
+        self.assertEqual(token[0].xsd_types['{http://xpath.test/ns}values'],
+                         schema.elements['values'].type)
         self.assertEqual(token[1].xsd_types['c'], schema.maps.types['{%s}boolean' % XSD_NAMESPACE])
         token = parser.parse("//b/@min")
         self.assertEqual(token[0][0].xsd_types['b'], schema.types['rangeType'])
@@ -293,7 +317,8 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
                          schema.maps.types['{%s}integer' % XSD_NAMESPACE])
 
         token = parser.parse("values/b/@min")
-        self.assertEqual(token[0][0].xsd_types['values'], schema.elements['values'].type)
+        self.assertEqual(token[0][0].xsd_types['{http://xpath.test/ns}values'],
+                         schema.elements['values'].type)
         self.assertEqual(token[0][1].xsd_types['b'], schema.types['rangeType'])
         self.assertEqual(token[1][0].xsd_types['min'],
                          schema.maps.types['{%s}integer' % XSD_NAMESPACE])
