@@ -25,6 +25,7 @@ import io
 import math
 import pickle
 from decimal import Decimal
+from typing import Optional, List, Tuple
 from xml.etree import ElementTree
 
 try:
@@ -303,9 +304,23 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
                 self.parser.schema = None
 
     def test_check_variables_method(self):
-        self.assertIsNone(self.parser.check_variables(
-            {'values': [1, 2, -1], 'myaddress': 'info@example.com', 'word': ''}
-        ))
+        self.assertIsNone(self.parser.check_variables({
+            'values': [1, 2, -1],
+            'myaddress': 'info@example.com',
+            'word': ''
+        }))
+
+        with self.assertRaises(TypeError) as ctx:
+            self.parser.check_variables({'values': [None, 2, -1]})
+        error_message = str(ctx.exception)
+        self.assertIn('XPDY0050', error_message)
+        self.assertIn('Unmatched sequence type', error_message)
+
+        with self.assertRaises(TypeError) as ctx:
+            self.parser.check_variables({'other': None})
+        error_message = str(ctx.exception)
+        self.assertIn('XPDY0050', error_message)
+        self.assertIn('Unmatched sequence type', error_message)
 
     # XPath expression tests
     def test_node_selection(self):
@@ -366,9 +381,11 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.wrong_syntax("{%s" % XSD_NAMESPACE)
         self.wrong_syntax("{%s}1" % XSD_NAMESPACE)
         self.check_value("{%s}true()" % XPATH_FUNCTIONS_NAMESPACE, True)
+        self.check_value("string({%s}true())" % XPATH_FUNCTIONS_NAMESPACE, 'true')
 
+        context = XPathContext(root)
         name = '{%s}alpha' % XPATH_FUNCTIONS_NAMESPACE
-        self.check_value(name, name)  # it's not an error to use 'fn' namespace for a name
+        self.check_value(name, [], context)  # it's not an error to use 'fn' namespace for a name
 
         self.parser.strict = True
         self.wrong_syntax('{%s}string' % XSD_NAMESPACE)
@@ -436,6 +453,9 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         context.item = 1
         self.check_value("self::node()", expected=[], context=context)
 
+    def test_unknown_function(self):
+        self.wrong_type("unknown('5')", 'XPST0017', 'unknown function')
+
     def test_node_set_id_function(self):
         # XPath 1.0 id() function: https://www.w3.org/TR/1999/REC-xpath-19991116/#function-id
         root = self.etree.XML('<A><B1 xml:id="foo"/><B2/><B3 xml:id="bar"/><B4 xml:id="baz"/></A>')
@@ -670,8 +690,10 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.wrong_syntax("starts-with((), ())")
             self.check_value("starts-with('1999', 19)", True)
         else:
-            self.check_value('fn:starts-with("tattoo", "tat")', True)
-            self.check_value('fn:starts-with ( "tattoo", "att")', False)
+            self.check_value('fn:starts-with("tattoo", "tat", "http://www.w3.org/'
+                             '2005/xpath-functions/collation/codepoint")', True)
+            self.check_value('fn:starts-with ("tattoo", "att", "http://www.w3.org/'
+                             '2005/xpath-functions/collation/codepoint")', False)
             self.check_value('fn:starts-with ((), ())', True)
             self.wrong_type("starts-with('1999', 19)")
             self.parser.compatibility_mode = True
@@ -722,8 +744,10 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.wrong_syntax("contains((), ())")
             self.check_value("contains('XPath', 20)", False)
         else:
-            self.check_value('fn:contains ( "tattoo", "t")', True)
-            self.check_value('fn:contains ( "tattoo", "ttt")', False)
+            self.check_value('fn:contains ( "tattoo", "t", "http://www.w3.org/'
+                             '2005/xpath-functions/collation/codepoint")', True)
+            self.check_value('fn:contains ( "tattoo", "ttt", "http://www.w3.org/'
+                             '2005/xpath-functions/collation/codepoint")', False)
             self.check_value('fn:contains ( "", ())', True)
             self.wrong_type("contains('XPath', 20)")
             self.parser.compatibility_mode = True
@@ -748,9 +772,13 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_value("substring-before('2017-10-27', 10)", '2017-')
             self.wrong_syntax("fn:substring-before((), ())")
         else:
-            self.check_value('fn:substring-before ( "tattoo", "attoo")', 't')
-            self.check_value('fn:substring-before ( "tattoo", "tatto")', '')
+            self.check_value('fn:substring-before ( "tattoo", "attoo", "http://www.w3.org/'
+                             '2005/xpath-functions/collation/codepoint")', 't')
+            self.check_value('fn:substring-before ( "tattoo", "tatto", "http://www.w3.org/'
+                             '2005/xpath-functions/collation/codepoint")', '')
+
             self.check_value('fn:substring-before ((), ())', '')
+            self.check_value('fn:substring-before ((), "")', '')
             self.wrong_type("substring-before('2017-10-27', 10)")
             self.parser.compatibility_mode = True
             self.check_value("substring-before('2017-10-27', 10)", '2017-')
@@ -854,6 +882,9 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_selector('lang("en")', root, True)
         if self.parser.version > '1.0':
             self.check_selector('para/lang("en")', root, True)
+            context = XPathContext(root)
+            self.check_value('for $x in . return $x/fn:lang(())',
+                             expected=[], context=context)
         else:
             context = XPathContext(document, item=root[0])
             self.check_value('lang("en")', True, context=context)
@@ -863,6 +894,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_selector('lang("en")', root, False)
         if self.parser.version > '1.0':
             self.check_selector('b/c/lang("en")', root, False)
+            self.check_selector('b/c/lang("en", .)', root, False)
         else:
             context = XPathContext(root, item=root[0][0])
             self.check_value('lang("en")', False, context=context)
@@ -874,10 +906,13 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
         document = self.etree.ElementTree(root)
         context = XPathContext(root=document)
-        if self.parser.version > '1.0':
-            self.check_value('lang("en")', expected=TypeError, context=context)
-        else:
+        if self.parser.version == '1.0':
             self.check_value('lang("en")', expected=False, context=context)
+        else:
+            self.check_value('lang("en")', expected=TypeError, context=context)
+            context.item = document
+            self.check_value('for $x in /a/b/c return $x/fn:lang("en")',
+                             expected=[False], context=context)
 
     def test_logical_and_operator(self):
         self.check_value("false() and true()", False)
@@ -918,6 +953,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value("5 > 3", True)
         self.check_value("5 < 20.0", True)
         self.check_value("2 * 2 = 4", True)
+        self.wrong_syntax("5 > 3 < 4", "unexpected '<' operator")
 
         if self.parser.version == '1.0':
             self.check_value("false() = 1", False)
@@ -925,6 +961,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         else:
             self.wrong_type("false() = 1")
             self.wrong_type("0 = false()")
+            self.wrong_value('xs:untypedAtomic("1") = xs:dayTimeDuration("PT1S")',
+                             'FORG0001', "'1' is not an xs:duration value")
 
     def test_comparison_of_sequences(self):
         root = self.etree.XML('<table>'
@@ -948,12 +986,34 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value("8 - 5", 3)
         self.check_value("-8 - 5", -13)
         self.check_value("-3 * 7", -21)
-        self.check_value("9 - 1 + 6", 14)
         self.check_value("(5 * 7) + 9", 44)
         self.check_value("-3 * 7", -21)
         self.check_value('(2 + 4) * 5', 30)
         self.check_value('2 + 4 * 5', 22)
 
+    def test_addition_and_subtraction_operators(self):
+        # '+' and '-' are both prefix and infix operators. The binding
+        # power is equal to 40 but the nud() method is set with rbp=70.
+        self.check_value("9 + 1 + 6", 16)
+        self.check_tree("9 - 1 + 6", '(+ (- (9) (1)) (6))')
+        self.check_value("(9 - 1) + 6", 14)
+        self.check_value("9 - 1 + 6", 14)
+
+        self.check_tree('1 + 2 * 4 + (1 + 2 + 3 * 4)',
+                        '(+ (+ (1) (* (2) (4))) (+ (+ (1) (2)) (* (3) (4))))')
+        self.check_value('1 + 2 * 4 + (1 + 2 + 3 * 4)', 24)
+
+        self.check_tree('15 - 13.64 - 1.36', "(- (- (15) (Decimal('13.64'))) (Decimal('1.36')))")
+        self.check_tree('15 + 13.64 + 1.36', "(+ (+ (15) (Decimal('13.64'))) (Decimal('1.36')))")
+        self.check_value('15 - 13.64 - 1.36', 0)
+
+        if self.parser.version != '1.0':
+            self.check_tree('(5, 6) instance of xs:integer+',
+                            '(instance (, (5) (6)) (: (xs) (integer)) (+))')
+            self.check_tree('- 1 instance of xs:int', "(instance (- (1)) (: (xs) (int)))")
+            self.check_tree('+ 1 instance of xs:int', "(instance (+ (1)) (: (xs) (int)))")
+            self.wrong_type('2 - 1 instance of xs:int', 'XPTY0004')
+
     def test_div_operator(self):
         self.check_value("5 div 2", 2.5)
         self.check_value("0 div 2", 0.0)
@@ -961,6 +1021,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_value("10div 3", SyntaxError)  # TODO: accepted syntax in XPath 1.0
         else:
             self.check_value("() div 2")
+            self.check_raise('1 div 0.0', ZeroDivisionError, 'FOAR0001', 'Division by zero')
 
     def test_numerical_add_operator(self):
         self.check_value("3 + 8", 11)
@@ -979,6 +1040,10 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_value("/values/b + 2", ValueError, context=XPathContext(root))
             self.wrong_type("+'alpha'")
             self.wrong_type("3 + 'alpha'")
+            self.check_value("() + 81")
+            self.check_value("72 + ()")
+            self.check_value("+()")
+            self.wrong_type('xs:dayTimeDuration("P1D") + xs:duration("P6M")', 'XPTY0004')
 
         self.check_selector("/values/d + 3", root, 47)
 
@@ -999,6 +1064,10 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_value("/values/b - 2", ValueError, context=XPathContext(root))
             self.wrong_type("-'alpha'")
             self.wrong_type("3 - 'alpha'")
+            self.check_value("() - 6")
+            self.check_value("19 - ()")
+            self.check_value("-()")
+            self.wrong_type('xs:duration("P3Y") - xs:yearMonthDuration("P2Y3M")', 'XPTY0004')
 
         self.check_selector("/values/d - 3", root, 41)
 
@@ -1006,6 +1075,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value("11 mod 3", 2)
         self.check_value("4.5 mod 1.2", Decimal('0.9'))
         self.check_value("1.23E2 mod 0.6E1", 3.0E0)
+        self.check_value("10 mod 0e1", math.isnan)
+        self.check_raise('3 mod 0', ZeroDivisionError, 'FOAR0001')
 
         root = self.etree.XML(XML_DATA_TEST)
         if self.parser.version == '1.0':
@@ -1014,6 +1085,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         else:
             self.check_selector("/values/a mod 2", root, TypeError)
             self.check_value("/values/b mod 2", TypeError, context=XPathContext(root))
+            self.check_value("() mod 2e1")
+            self.check_value("2 mod xs:float('INF')", 2)
 
         self.check_selector("/values/d mod 3", root, 2)
 
@@ -1331,6 +1404,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_selector('/A/*/C2/parent::node()', root, [root[2]])
         self.check_selector('/A/*/*/parent::node()', root, [root[0], root[2], root[3]])
         self.check_selector('//C2/parent::node()', root, [root[2]])
+
+        self.check_selector('..', self.etree.ElementTree(root), [])
         self.check_value('..', MissingContextError)
         self.check_value('parent::*', MissingContextError)
 
@@ -1422,6 +1497,9 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value("a[not(b)]", [], context=XPathContext(root, item=root[0]))
         self.check_value("a[not(b)]", [root[1][0]], context=XPathContext(root, item=root[1]))
 
+        self.check_raise('88[..]', TypeError, 'XPTY0020', 'Context item is not a node',
+                         context=XPathContext(root))
+
         self.check_tree("preceding::a[not(b)]", '([ (preceding (a)) (not (b)))')
 
         self.check_value("a[preceding::a[not(b)]]", [], context=XPathContext(root, item=root[0]))
@@ -1441,6 +1519,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_selector('/A/B2 | /A/*', root, root[:])
         self.check_selector('/A/B2 | /A/* | /A/B1', root, root[:])
         self.check_selector('/A/@min | /A/@max', root, {'1', '10'})
+        self.check_raise('1|2|3', TypeError, 'XPTY0004', 'only XPath nodes are allowed',
+                         context=XPathContext(root))
 
     def test_default_namespace(self):
         root = self.etree.XML('<foo>bar</foo>')
@@ -1540,11 +1620,12 @@ class LxmlXPath1ParserTest(XPath1ParserTest):
 
     def test_namespace_axis(self):
         root = self.etree.XML('<A xmlns:tst="http://xpath.test/ns"><tst:B1/></A>')
-        namespaces = list(self.parser.DEFAULT_NAMESPACES.items()) \
-            + [('tst', 'http://xpath.test/ns')]
+        namespaces: List[Tuple[Optional[str], str]] = []
+        namespaces.extend(self.parser.DEFAULT_NAMESPACES.items())
+        namespaces += [('tst', 'http://xpath.test/ns')]
+
         self.check_selector('/A/namespace::*', root, expected=set(namespaces),
                             namespaces=namespaces[-1:])
-
         self.check_selector('/A/namespace::*', root, expected=set(namespaces))
 
         root = self.etree.XML('<tst:A xmlns:tst="http://xpath.test/ns" '
