@@ -33,6 +33,14 @@ except ImportError:
 
 class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
 
+    def test_unknown_constructor(self):
+        self.wrong_type("xs:unknown('5')", 'XPST0017', 'unknown constructor function')
+
+    def test_invalid_arguments(self):
+        # Invalid argument types (parsed by null-denotation method)
+        self.wrong_type('xs:normalizedString(()', 'XPST0017')
+        self.wrong_type('xs:normalizedString(5, 2)', 'XPST0017')
+
     def test_string_constructor(self):
         self.check_value("xs:string(5.0)", '5')
         self.check_value("xs:string(5.2)", '5.2')
@@ -41,6 +49,9 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:string(())', [])
         self.wrong_syntax('xs:string(()', 'XPST0017')
 
+        # canonical string representation of xs:hexBinary
+        self.check_value('xs:string(xs:hexBinary("ef"))', 'EF')
+
     def test_normalized_string_constructor(self):
         self.check_value('xs:normalizedString("hello")', "hello")
         self.check_value('xs:normalizedString(" hello  ")', " hello  ")
