@@ -21,8 +21,10 @@
 #
 import unittest
 import io
+import locale
 import os
 from decimal import Decimal
+from textwrap import dedent
 
 try:
     import lxml.etree as lxml_etree
@@ -344,11 +346,6 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
                    root[2][0], root[2][2], root[2][0], root[2][3], root[2][0]]
         )
 
-    def test_numerical_add_operator(self):
-        super(XPath2ParserTest, self).test_numerical_add_operator()
-        self.check_value("() + 81")
-        self.check_value("72 + ()")
-
     def test_idiv_operator(self):
         self.check_value("5 idiv 2", 2)
         self.check_value("-3.5 idiv -2", 1)
@@ -369,6 +366,8 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_value("false() eq 1", False)
         self.check_value("0 eq false()", True)
         self.check_value("2 * 2 eq 4", True)
+        self.check_value("() * 7")
+        self.check_value("() * ()")
 
         self.check_value("() le 4")
         self.check_value("4 gt ()")
@@ -468,9 +467,8 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
                 self.check_value('. le 20', True, context=XPathContext(root))
 
                 root = self.etree.XML('<root>eleven</root>')
-                with self.assertRaises(TypeError) as err:
-                    self.check_value('. le 10', context=XPathContext(root))
-                self.assertIn('XPTY0004', str(err.exception))  # Dynamic context error
+                self.wrong_type('. le 10', 'XPDY0050', 'does not match sequence type',
+                                context=XPathContext(root))
 
                 root = self.etree.XML('<value>12</value>')
                 with self.assertRaises(TypeError) as err:
@@ -607,6 +605,14 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_value('xs:time("08:20:00-05:00") - xs:dayTimeDuration("P23DT10H10M")',
                          Time.fromstring('22:10:00-05:00'))
 
+    def test_duration_with_arithmetical_operators(self):
+        self.wrong_type('xs:duration("P1Y") * 3', 'XPTY0004', 'unsupported operand type(s)')
+        self.wrong_value('xs:duration("P1Y") * xs:float("NaN")', 'FOCA0005')
+        self.check_value('xs:duration("P1Y") * xs:float("INF")', OverflowError)
+        self.wrong_value('xs:float("NaN") * xs:duration("P1Y")', 'FOCA0005')
+        self.check_value('xs:float("INF") * xs:duration("P1Y")', OverflowError)
+        self.wrong_type('xs:duration("P3Y") div 3',  'XPTY0004', 'unsupported operand type(s)')
+
     def test_year_month_duration_operators(self):
         self.check_value('xs:yearMonthDuration("P2Y11M") + xs:yearMonthDuration("P3Y3M")',
                          YearMonthDuration(months=74))
@@ -617,6 +623,13 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_value('xs:yearMonthDuration("P2Y11M") div 1.5',
                          YearMonthDuration.fromstring('P1Y11M'))
         self.check_value('xs:yearMonthDuration("P3Y4M") div xs:yearMonthDuration("-P1Y4M")', -2.5)
+        self.wrong_value('xs:double("NaN") * xs:yearMonthDuration("P2Y")', 'FOCA0005')
+        self.check_value('xs:yearMonthDuration("P1Y") * xs:double("INF")', OverflowError)
+        self.wrong_value('xs:yearMonthDuration("P3Y") div xs:double("NaN")', 'FOCA0005')
+
+        self.check_raise('xs:yearMonthDuration("P3Y") div xs:yearMonthDuration("P0Y")',
+                         ZeroDivisionError, 'FOAR0001', 'Division by zero')
+        self.check_raise('xs:yearMonthDuration("P3Y36M") div 0', OverflowError, 'FODT0002')
 
     def test_day_time_duration_operators(self):
         self.check_value('xs:dayTimeDuration("P2DT12H5M") + xs:dayTimeDuration("P5DT12H")',
@@ -627,6 +640,8 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
                          DayTimeDuration.fromstring('PT4H33M'))
         self.check_value('xs:dayTimeDuration("P1DT2H30M10.5S") div 1.5',
                          DayTimeDuration.fromstring('PT17H40M7S'))
+        self.check_value('3 * xs:dayTimeDuration("P1D")',
+                         DayTimeDuration.fromstring('P3D'))
         self.check_value(
             'xs:dayTimeDuration("P2DT53M11S") div xs:dayTimeDuration("P1DT10H")',
             Decimal('1.437834967320261437908496732')
@@ -921,7 +936,7 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
 
     @unittest.skipIf(xmlschema is None, "xmlschema library required.")
     def test_get_atomic_value(self):
-        schema = xmlschema.XMLSchema("""
+        schema = xmlschema.XMLSchema(dedent("""\
             <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
               <xs:element name="a" type="aType"/>
               <xs:complexType name="aType">
@@ -943,7 +958,7 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
               <xs:simpleType name="eType">
                 <xs:union memberTypes="xs:string xs:integer xs:boolean"/>
               </xs:simpleType>
-            </xs:schema>""")
+            </xs:schema>"""))
 
         token = self.parser.parse('true()')
 
@@ -960,7 +975,7 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
 
         self.parser.schema = xmlschema.xpath.XMLSchemaProxy(schema)
         try:
-            with self.assertRaises(AttributeError) as err:
+            with self.assertRaises(AttributeError):
                 self.parser.get_atomic_value(schema)
 
             value = self.parser.get_atomic_value('unknown')
@@ -994,6 +1009,91 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         finally:
             self.parser.schema = None
 
+    def test_auxiliary_tokens(self):
+        self.check_raise('as', MissingContextError)
+        self.check_raise('of', MissingContextError)
+
+        context = XPathContext(self.etree.XML('<root/>'))
+        self.check_raise('as', MissingContextError, context=context)
+        self.check_raise('of', MissingContextError, context=context)
+
+    def test_function_namespace(self):
+        function_namespace = "http://xpath.test/fn/xpath-functions"
+        parser = self.parser.__class__(
+            namespaces={'fn2': function_namespace},
+            function_namespace=function_namespace
+        )
+        token = parser.parse('fn2:true()')
+        self.assertTrue(token.evaluate())
+
+    def test_invalid_schema_argument(self):
+        schema = dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+              <xs:element name="root" />
+            </xs:schema>""")
+
+        with self.assertRaises(ElementPathTypeError) as ctx:
+            self.parser.__class__(schema=schema)
+        self.assertEqual(str(ctx.exception),
+                         "argument 'schema' must be an instance of AbstractSchemaProxy")
+
+        if xmlschema is not None:
+            with self.assertRaises(ElementPathTypeError):
+                self.parser.__class__(schema=xmlschema.XMLSchema(schema))
+
+    def test_variable_types_argument(self):
+        variable_types = {'a': 'item()', 'b': 'xs:integer'}
+        parser = self.parser.__class__(variable_types=variable_types)
+        self.assertEqual(variable_types, parser.variable_types)
+        self.assertIsNot(variable_types, parser.variable_types)
+
+        with self.assertRaises(ElementPathValueError) as ctx:
+            self.parser.__class__(variable_types={'a': 'item()', 'b': 'xs:complex'})
+        self.assertEqual(str(ctx.exception),
+                         "invalid sequence type for in-scope variable types")
+
+    def test_document_types_argument(self):
+        document_types = {'doc1': 'node()*', 'doc2': 'element()'}
+        parser = self.parser.__class__(document_types=document_types)
+        self.assertEqual(document_types, parser.document_types)
+        self.assertIs(document_types, parser.document_types)
+
+        with self.assertRaises(ElementPathValueError) as ctx:
+            self.parser.__class__(document_types={'doc1': 'node()*', 'doc2': 'etree()'})
+        self.assertEqual(str(ctx.exception),
+                         "invalid sequence type in document_types argument")
+
+    def test_collection_types_argument(self):
+        collection_types = {'col1': 'node()*', 'col2': 'element()*'}
+        parser = self.parser.__class__(collection_types=collection_types)
+        self.assertEqual(collection_types, parser.collection_types)
+        self.assertIs(collection_types, parser.collection_types)
+
+        with self.assertRaises(ElementPathValueError) as ctx:
+            self.parser.__class__(collection_types={'doc1': 'node()*', 'doc2': 'etree()*'})
+        self.assertEqual(str(ctx.exception),
+                         "invalid sequence type in collection_types argument")
+
+    def test_default_collection_type_argument(self):
+        parser = self.parser.__class__(default_collection_type='element()*')
+        self.assertEqual(parser.default_collection_type, 'element()*')
+
+        with self.assertRaises(ElementPathValueError) as ctx:
+            self.parser.__class__(default_collection_type='elem()*')
+        self.assertEqual(str(ctx.exception),
+                         "invalid sequence type for default_collection_type argument")
+
+    def test_default_collation_argument(self):
+        default_locale = locale.getdefaultlocale()
+        collation = '.'.join(default_locale) if default_locale[1] else default_locale[0]
+
+        if collation == 'en_US.UTF-8':
+            collation = "http://www.w3.org/2005/xpath-functions/collation/codepoint"
+        self.assertEqual(self.parser.__class__().default_collation, collation)
+
+        parser = self.parser.__class__(default_collation='it_IT.UTF-8')
+        self.assertEqual(parser.default_collation, 'it_IT.UTF-8')
+
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
 class LxmlXPath2ParserTest(XPath2ParserTest):
